;====================
; DXBCN project
; Controller of HAM radio beacon
; (C) Pavel Vachal,OK1DX 2012
; version 2.0 AtMega8 preliminary 22.12.2012
; compiled in AVR Studio version 4
; 
; HAMWARE - free for Amateur radio community
; commercial use only with author's approval
;====================
; history: initial release 1.0;
;          version 2 - moved to AtMega8, added WSPR, GPS data

; ToDo:		pouziti signalu 1Hz clock z GPS, zahrnuti udaje o msec z GPS msg
; vsechny implementovane funkce otestovane a funkcni!!!!!!

.NOLIST

;*********  FUNCTION LIST  *********
; LOOP	- main program loop, started after initialisation
; AD985X - sends frequency value to DDS chip
; RD_EE - reads one byte from EEPROM
; EE_BLK - read data block from EEPROM
; STXT - sets CW text pointer to begin of selected text

;******** INTERRUPT HANDLERS *******
; RESET - entry point
; TIM0_OVF - timer 0 overflow: handling of CW timing
; TIM1_OVF - timer 1 overflow: handling of WSPR timing
; IRQ0X - INT0 external interrupt 1 Hz GPS clock

;******** FUSES ********************
; ATmega8 needs to program fuses to operates correctly with external Xtal resonator
; BODLEVEL = 1, BODEN = 1, SUT1 = 1, SUT0 = 1, CKSEL3 = 1, CKSEL2 = 1, CKSEL1 = 1, CKSEL0 = 1 -> lower fuse byte = 11111111
; higher fuse byte = 11111111 (default)
; fuses programming must be done separately (not performed during code programming)!!!!

.include	"m8def.inc"
.LIST
;.SET AD9850	=1	; rem when using AD9851 with prescaller

; port initialisation
; DDR bit 1=output, 0=input
; PORT bit when input 1=pullup 0=tri state
; Xtal freq = 10.00000 MHz (important)

; PORT B:  PB0 spare input, PB1 mode selector, PB2 spare input, PB3-5 programmer, PB6,7 Xtal
.EQU	_DDRB	=0x0	; PB0-5 input pullup, PB6,7 input
.EQU	_PORTB	=0x3F

; port B bits
.EQU	MODESEL = 1

; PORT C: PC0 Sync LED, PC1 CW out, PC2 PTT, PC3-4 CW text selector, PC5 Sync Button, PC6 = Reset
.EQU	_DDRC	=0x07	; PC0-2 output, PC3-6 inputs pullup
.EQU	_PORTC	=0x78

; port C bits
.EQU SYNLED = 	0
.EQU CW_OUT = 	1
.EQU PTT = 		2
.EQU CWTSEL0 = 	3
.EQU CWTSEL1 = 	4
.EQU SYNCB = 	5


; PORT D: PD0 GPS data / USART, PD1 DdsData, PD2 GPS 1Hz (IRQ0), PD3 DdsClock, PD4 DdsFreqUpd, PD5-7 band data
.EQU	_DDRD	=0xFA	; PD0 input, PD1 output, PD2 input (temp pull up), PD3-7 output
.EQU	_PORTD	=0x04

; port D bits
.EQU DDS_DATA = 1
.EQU DDS_CLK = 3
.EQU DDS_FR_UP = 4

; registers
; R0 and R1 reserved for multiplications
.DEF	CWSPD   =R2	; speed constant from EEPROM
.DEF	CWCLK	=R3	; speed clock counter (up to CWSPD)
.DEF	ISREG	=R15	; temp SREG storage during processing of INT handler
.DEF	TMP1	=R16	; general temp regs used in main loop
.DEF	TMP2	=R17
.DEF	TMP3	=R18
.DEF	ITMP1	=R19 	; interrupt temporary register
.DEF	ITMP2	=R20 	; interrupt temporary register

.DEF	CWREG	=R23	; ********** CW register ************
.EQU	T0EXP	= 7		; bit 7 set when T0 timer expires
.EQU	ELSND	= 6		; bit 6 elem being sent
// bits 0-3 = counter

.DEF	WSREG	=R24	; ********** WSPR register **********
.EQU	STIME = 0		; bit 0 - symbol time expired flag
.EQU	DMIN = 1		; bit 1 - doubleminute flag
.EQU	SYNCBM = 2		; bit 2 - sync button memory (for edge detection)
.EQU	PWRDWN = 3		; bit 3 - power down (stop DDS) - general, not just WSPR!
.EQU	MODEWSPR = 4	; bit 4 - 1=active mode is WSPR, 0=CW

; X register (r26 LOW BYTE, r27 HIGH BYTE - always zero) used as pointer to freq data
; Y register (r28 LOW BYTE, r29 HIGH BYTE) pointer used to access EEPROM data

;********* MORSE CODE TABLE ********
.EQU	_A	=	0b110
.EQU	_B	=	0b10001
.EQU	_C	=	0b10101
.EQU	_D	=	0b1001
.EQU	_E	=	0b10
.EQU	_F	=	0b10100
.EQU	_G	=	0b1011
.EQU	_H	=	0b10000
.EQU	_I	=	0b100
.EQU	_J	=	0b11110
.EQU	_K	=	0b1101
.EQU	_L	=	0b10010
.EQU	_M	=	0b111
.EQU	_N	=	0b101
.EQU	_O	=	0b1111
.EQU	_P	=	0b10110
.EQU	_Q	=	0b11011
.EQU	_R	=	0b1010
.EQU	_S	=	0b1000
.EQU	_T	=	0b11
.EQU	_U	=	0b1100
.EQU	_V	=	0b11000
.EQU	_W	=	0b1110
.EQU	_X	=	0b11001
.EQU	_Y	=	0b11101
.EQU	_Z	=	0b10011
.EQU	_0	=	0b111111
.EQU	_1	=	0b111110
.EQU	_2	=	0b111100
.EQU	_3	=	0b111000
.EQU	_4	=	0b110000
.EQU	_5	=	0b100000
.EQU	_6	=	0b100001
.EQU	_7	=	0b100011
.EQU	_8	=	0b100111
.EQU	_9	=	0b101111
.EQU	_SL	=	0b101001	; /
.EQU	_AR	=	0b101010
.EQU	_SK	=	0b1101000
.EQU	_PT	=	0b1101010	; .
.EQU	_CM	=	0b1110011	; ,
.EQU	_EQ	=	0b110001	; =

.EQU	_SPC	=	1		; space
.EQU	_TEND	=	0	; end of text

.DSEG
; static RAM registers
CWPTRL: 	.BYTE 1	; EEPROM CW text pointer, low byte address
CWPTRH: 	.BYTE 1	; EEPROM CW text pointer, high byte address
CWCHAR: 	.BYTE 1	; character being sent
FREQ1:		.BYTE 4	; freq + control, key up / 4 bytes
FREQ2:		.BYTE 4	; freq + control, key down / 4 bytes
WSCTR:		.BYTE 1	; WSPR symbol counter (incremented every 0.6826667 sec)
WSBYTE:		.BYTE 1	; WSPR data byte (4x2bits), shifted
WPTRL:		.BYTE 1	; WPSR message pointer, low byte (pointing to EEPROM, next byte to be retrived)
WPTRH:		.BYTE 1	; WPSR message pointer, high byte
WSBFRQ:		.BYTE 4	; WSPR basic freq + control / 4 bytes
WSSFT:		.BYTE 1	; freq shift for WSPR in DDS increments (for 1.46 Hz result)
WSFREQ:		.BYTE 4	; WSPR actual freq (calculated from base + shift) / 4 bytes
FTCTR:		.BYTE 1	; freq table counter - incremented every 2 minutes, synced at whole hour. 0 to FTSIZE-1
FTSIZE:		.BYTE 1	; freq table size (initialized from EEPROM)
CYCLE1L:	.BYTE 1	; timer 1 preset - symbol time, LSB
CYCLE1H:	.BYTE 1	; timer 1 preset - symbol time, MSB
CYCLE2L:	.BYTE 1	; timer 1 preset - tail time, LSB
CYCLE2H:	.BYTE 1	; timer 1 preset - tail time, MSB
GCTR:		.BYTE 1	; GPS counter - counts characters in $GPGGA sequence
GBUF:		.BYTE 9 ; GPS msg buffer / 9 bytes (time in format HHMMSS.ss)
.EQU		GBUFL = 9	; buffer length


.CSEG
.ORG	0

; interrupt vector table	
	rjmp	RESET		;1	Reset Handler
	rjmp	IRQ0X 		;2 IRQ0 Handler
	reti		;3 IRQ1 Handler
	reti		;4 TIMER2_COMP
	reti		;5 TIMER2_OVF
	reti		;6 rjmp TIM1_CAPT ; Timer1 Capture Handler
	reti		;7 rjmp TIM1_COMPA ; Timer1 compare Handler
	reti		;8 rjmp TIM1_COMPA ; Timer1 compare Handler
	rjmp	TIM1_OVF ;9 Timer1 Overflow Handler
	rjmp	TIM0_OVF ;10 Timer0 Overflow Handler
	reti		;11 SPI Transfer Complete Handler
	reti		;12 UART RX Complete Handler
	reti 		;13 UDR Empty Handler
	reti		;14 rjmp UART_TXC ; UART TX Complete Handler
	reti   		;15 ADC ; ADC Conversion Complete Interrupt Handler
	reti		;16 EEPROM Ready Handler
	reti		;17 Analog Comparator Handler
	reti		;18 TWI
	reti		;19 SPM_RDY

GHEAD:
.DB '$','G','P','G','G','A',',',0x0
.EQU GHLEN = 7	; GHEADER length

RESET:
	ldi	TMP1,0x04	; initiate stack pointer to 0x45F / RAM top
	out	SPH,TMP1
	ldi	TMP1,0x5F
	out	SPL,TMP1

	ldi	TMP1,_DDRC	; port mode ini
	out	DDRC,TMP1
	ldi	TMP1,_DDRB
	out	DDRB,TMP1
	ldi	TMP1,_DDRD
	out	DDRD,TMP1


	ldi	TMP1,_PORTC
	out	PORTC,TMP1
	ldi	TMP1,_PORTB
	out	PORTB,TMP1
	ldi	TMP1,_PORTD
	out	PORTD,TMP1

; pulse on FR_UP / activate serial data transfer mode
	sbi	PORTD,DDS_FR_UP	; UPDATE FREQ pulse
	ldi	TMP2,6
RESET1:	dec	TMP2	; delay 4 microsec
	brne	RESET1	
	cbi	PORTD,DDS_FR_UP

; CPU register INI
	clr	CWREG
	rcall	STXT		; select CW text to be TXed
	ldi	TMP1,1
	sts	CWCHAR,TMP1

; USART INIT - RX only, no interrupt (polling mode)
	ldi TMP1,0x86	; async 8bit no parity 1 stop bit (identical with default, but...)
	out UCSRC,TMP1
	clr	TMP1
	out UBRRH,TMP1	; baud divider = 129 -> 4800 Bd
	ldi TMP1,129
	out UBRRL,TMP1
	ldi TMP1,0x10	; receiver enable
	out UCSRB,TMP1

; read parameters from EEPROM
	ldi XH, HIGH(CYCLE1L)	; CYCLE1 addr in SRAM (CYCLE2 follows)
	ldi	XL, LOW(CYCLE1L)
	ldi	TMP3, 4				; 4 bytes total
	ldi	YL, LOW(ECYCLE1)	; pointer to EEPROM
	ldi YH, HIGH(ECYCLE1)
	rcall EE_BLK

	ldi	YL,LOW(ECWSPD)
	ldi YH,HIGH(ECWSPD)
	rcall	RD_EE
	mov	CWSPD, TMP1	; CW speed constant
	mov	CWCLK, TMP1

	ldi	YL,LOW(EWSHFT)
	ldi YH,HIGH(EWSHFT)
	rcall	RD_EE
	sts	WSSFT, TMP1	; WSPR shift

	ldi	YL,LOW(EFTSIZE)
	ldi YH,HIGH(EFTSIZE)
	rcall	RD_EE
	sts	FTSIZE, TMP1	; freq table size

	ldi TMP1, HIGH(WSPRM)	; set message pointer to start of message
	sts	WPTRH, TMP1
	ldi TMP1, LOW(WSPRM)
	sts WPTRL, TMP1

	clr TMP1
	sts GCTR, TMP1				; GPS message counter clear
	sts WSCTR, TMP1				; cycle counter clear
	dec TMP1
	sts FTCTR, TMP1				; freq table counter to very high value (will be set to 0 immy)

; INITIALISATION - REGISTERS
	ldi WSREG, 1<<STIME	; in order to perform reading of freq table immy

; TIMER 0 initialisation
	ldi	TMP1,3
	out	TCCR0, TMP1	; T0 8 bit internal/64	

; TIMER 1 initialisation
	ldi	TMP1,4
	out	TCCR1B, TMP1	; T1 16bit internal/256
	ldi TMP1, 100
	out TCNT1H, TMP1
	out TCNT1L, TMP1	; very first call, not important timing

	ldi	TMP1,5
	out	TIMSK, TMP1		; enable TOIE0, TOEI1 interrupt only

;IRQ0 initialisation
	ldi	TMP1,3			; rising edge IRQ0 interrupt; IRQ1 disabled, sleep mode disabled
	out MCUCR,TMP1
	ldi TMP1,1<<INT0
	out GICR,TMP1		; enable IRQ0

	sei			; enable global interrupt

; ****** main program loop / performs most of the work
LOOP:
		sbrs WSREG, STIME
		rjmp FT4
		cbr WSREG,1<<STIME		; clear the flag
		lds TMP1,WSCTR
		cpi TMP1,0
		breq	FT3		; no doubleminute end, proceed further
		rjmp FT5
FT3:	lds TMP1,FTCTR	; end of doubleminute. Process next entry in freq table
		inc TMP1
		lds TMP2, FTSIZE
		cp TMP1, TMP2	; if FTPTR >= FTSIZE, set FTPTR = 0
		brcs FT1
		clr TMP1
FT1:	sts FTCTR, TMP1	; store new FTPTR value
		mov TMP2, TMP1
		add TMP1,TMP1	; TMP1 = 2* FTCTR
		add TMP1,TMP1	; TMP1 = 4* FTCTR
		add	TMP1,TMP2	; TMP1 = 5* FTCTR (offset in freq table is 5 bytes)
		ldi TMP2,LOW(FTABLE)
		add TMP1,TMP2
		mov YL,TMP1
		ldi TMP2,HIGH(FTABLE)
		clr TMP1
		adc TMP1,TMP2
		mov YH, TMP1	; Y = address in EEPROM of table entry
		rcall RD_EE
		adiw Y,1		; prepare pointer to freq data
		mov TMP2, TMP1
		ror TMP2
		ror TMP2
		ror TMP2
		ror TMP2
		andi TMP2,0xE0	; mask bits 5-7 (band data after shift)
		ori	TMP2,_PORTD		; add pull up for inputs 
		out	PORTD, TMP2
		sbrc TMP1,7		; bit 7 - selector CW/WSPR
		rjmp FT2		; jump - WSPR mode

; **** INITIALIZE PARAMETERS FOR CW DOUBLEMINUTE
		cbr WSREG, 1<<MODEWSPR	; set CW mode flag
		sbi PORTC,PTT			; activate PTT line
		ldi XL, LOW(FREQ1)
		ldi XH, HIGH(FREQ1)
		ldi TMP3, 4		; read 4 bytes
		rcall EE_BLK	; read CW FREQ1
		ldi YL,LOW(ECWSFT)
		ldi YH,HIGH(ECWSFT)
		rcall RD_EE		; TMP1 = shift LSB
		lds TMP2, FREQ1
		add TMP1,TMP2
		sts FREQ2, TMP1
		adiw Y,1
		rcall RD_EE		; TMP1 = shift MSB
		lds TMP2, FREQ1+1
		adc TMP1,TMP2
		sts FREQ2+1, TMP1
		clr TMP1
		lds TMP2, FREQ1+2
		adc TMP1, TMP2
		sts FREQ2+2, TMP1
		lds TMP2, FREQ1+3
		clr TMP1
		adc TMP1, TMP2
		sts FREQ2+3, TMP1		; FREQ2 = FREQ1 + shift
		cbr WSREG,1<<STIME		; clear the flag (only for CW, WSPR clears it later
		rjmp CW

; **** INITIALIZE PARAMETERS FOR WSPR DOUBLEMINUTE
FT2:	sbr WSREG, 1<<MODEWSPR	; set WSPR mode flag
		cbi PORTC,CW_OUT		; deactivate CW key line 
		ldi XL, LOW(WSBFRQ)
		ldi XH, HIGH(WSBFRQ)
		ldi TMP3, 4
		rcall EE_BLK	; read WSPR basic freq into WSBFRQ
		ldi TMP1, HIGH(WSPRM)	; set message pointer to start of message
		sts	WPTRH, TMP1
		ldi TMP1, LOW(WSPRM)
		sts WPTRL, TMP1
		rjmp WSPR

; STIME not set: CW processed, WSPR bypassed
FT4:	sbrc WSREG,MODEWSPR		; bit set = WSPR (bypassed, go to GPS instead)
		rjmp GPS
		rjmp CW

; STIME set (but WSCTR not zero). CW proceed, WSPR processed			
FT5:	sbrc WSREG,MODEWSPR		; bit set = WSPR 	
		rjmp WSPR

; CW message transmission
CW:		sbrs	CWREG,T0EXP			; CW time expired
		rjmp	LOOP2
		cbr	CWREG,1<<T0EXP		; clear the flag
		mov	TMP1,CWREG
		andi	TMP1, 0x0f		; get counter
		breq	CW1			; counter empty
		dec	CWREG
		rjmp 	LOOP2			; not yet empty, further wait
	
CW1:	sbrs	CWREG,ELSND			; ? element over, insert 1 space
		rjmp	CW2
		ldi	CWREG,0x00		; KEY DWN after dot or dash elem = 0, length = 1
		cbi	PORTC,CW_OUT		; KEY = up
		clr	r27
		ldi	r26,FREQ1
		rcall	AD985X			; set DDS to FREQ1
		rjmp	LOOP2
	
CW2:	lds	TMP1,CWCHAR
		cpi	TMP1,1
		breq	CW3			; ? send space after char
		cpi	TMP1,0
		breq	CW4			; ? take next char
	
		lsr	TMP1
		brcs	CW5
		ldi	CWREG,1<<ELSND		; send DOT , length = 1
		rjmp	CW6
CW5:	ldi	CWREG,(1<<ELSND) + 0x02		; send DASH , length = 3
CW6:	sbi	PORTC,CW_OUT		; KEY = down
		sts	CWCHAR,TMP1
		ldi	r27,HIGH(FREQ2)
		ldi	r26,LOW(FREQ2)
		cbr	WSREG,1<<PWRDWN		; always power ON
		rcall	AD985X			; set DDS to FREQ2
		rjmp	LOOP2
	
CW3:	ldi	CWREG,1<<ELSND		; SPACE AFTER CHAR elem = true, length = 1
		clr	TMP1
		sts	CWCHAR,TMP1
		rjmp	LOOP2
	
CW4:	clr	TMP2
		lds	YL,CWPTRL
		lds YH, CWPTRH
		rcall	RD_EE			; read one char from EEPROM to TMP1
		adiw Y,1			; increment text pointer
		sts	CWPTRH,YH
		sts CWPTRL,YL			; save new pointer
		sts	CWCHAR,TMP1
		cpi	TMP1,_TEND
		brne	CW2			; ? end of text
		rcall	STXT		; select start of CW text to be TXed
		rjmp	CW4			; read again from begining		
LOOP2:	rjmp GPS		; end of CW message transmission section


; ********** WSPR MAIN LOOP SECTION **********

WSPR:	lds TMP1,WSCTR
		andi TMP1, 0x03	; lower 2 bits (when cleared means WSCTR is multiple of 4)
		brne WS2
		lds YL,WPTRL	; we need to read next byte from EEPROM
		lds YH, WPTRH
		rcall RD_EE		; read symbol data from EEPROM
		adiw Y,1
		sts WPTRL,YL	; increment and save message pointer
		sts WPTRH,YH
		rjmp	WS3

WS2:	lds TMP1, WSBYTE
WS3:	clr TMP2		; symbol data shift
		lsl TMP1
		rol TMP2
		lsl TMP1
		rol TMP2	; 2 higher bits of TMP1 moved as 2 lower bits of TMP2
		sts WSBYTE,TMP1	; save shifted value
		lds TMP1,WSSFT
		mul TMP2, TMP1	; TMP2 = actual freq shift, result in r0:r1
		lds TMP1, WSBFRQ	; WSFREQ = WSBFRQ + shift
		add TMP1, r0
		sts	WSFREQ, TMP1
		lds TMP1, WSBFRQ+1
		adc	TMP1,r1
		sts WSFREQ+1, TMP1
		lds TMP1, WSBFRQ+2
		clr TMP2	; carry flag untouched
		adc	TMP1,TMP2
		sts WSFREQ+2, TMP1
		lds TMP1, WSBFRQ+3
		adc	TMP1,TMP2
		sts WSFREQ+3, TMP1

; power down / deactivate PTT and switch DDS to power down mode
		lds TMP1, WSCTR
		cpi	TMP1, 163		; 162 symbols
		brcs WS5			; no carry when WSCTR more than 162 -> stop TX!
		cbi	PORTC,PTT		; PTT deactivate
		sbr WSREG,1<<PWRDWN	; DDS power down (set bit to 1)
		rjmp WS6
WS5:	sbi	PORTC,PTT		; PTT activate
		cbr WSREG,1<<PWRDWN	; cancel DDS power down (clr bit to 0)

WS6:	ldi	XL,LOW(WSFREQ)	; send freq to DDS
		ldi XH,HIGH(WSFREQ)
		rcall AD985X


;********* GPS DATA PROCESSING SECTION ***************

GPS:	sbis UCSRA, RXC		; new received character
		rjmp SYN			; nothing to do, continue

		lds	TMP1,GCTR
		ldi TMP2,GHLEN
		sub	TMP1,TMP2
		brcc	GPS2		; header received completely, continue with the data saving
		ldi	ZH,HIGH(GHEAD<<1)	; GPGGA header in flash
		ldi ZL,LOW(GHEAD<<1)
		lds	TMP1,GCTR
		add	ZL,TMP1			; add counter
		clr TMP1
		adc ZH,TMP1
		lpm TMP1,Z
		in	TMP2,UDR		; byte received
		cp TMP1,TMP2
		brne	GPS5		; jump if character different from that expected (restart header search)
GPS3:	lds	TMP1,GCTR		; character match or char added to buffer, so increment counter
		inc TMP1
		sts	GCTR,TMP1
		rjmp	SYN
GPS2:	ldi XH,HIGH(GBUF)	; poinetr to buffer start (TMP1 = GCTR minus header length)
		ldi XL,LOW(GBUF)
		add XL,TMP1			; add offset in buffer
		clr TMP2
		adc XH,TMP2
		in TMP2,UDR			; char received
		st X,TMP2			; save it in buffer
		cpi TMP1,GBUFL-1	; buffer complete/full???
		brcs	GPS3		; the buffer is not yet complete, increment counter!
; when we get here, the buffer is full of data. Start processing!
		lds TMP1,GBUF+5		; ASCII character - seconds
		cpi TMP1,'0'
		brne	GPS5		; second digit different from 0 -> drop the telegram
		lds TMP1,GBUF+4
		cpi TMP1,'0'
		brne	GPS5		; tens second digit different from 0 -> drop the telegram
		lds TMP1,GBUF+3
		cpi TMP1,'9'
		brcc	GPS5		; minute digit > '8' -> drop the telegram
		cpi TMP1,'0'
		brcs	GPS5		; minute digit < '0' -> drop the telegram
		sbrc	TMP1,0		; test bit 0
		rjmp	GPS5		; minute digit is odd -> drop the telegram
; OK, the received time is begin of even minute -> lets synchronize the clock!
		brne	SYNC
		lds TMP1,GBUF+2		; minute digit == '0', load tens of minutes
		cpi TMP1,'0'
		brne	SYNC
		clr TMP1
		sts FTCTR,TMP1		; set Freq table counter to zero (start of a hour)
SYNC:	cli					; disable global IRQ
		sbr	WSREG,1<<DMIN	; set doubleminute flag (used later in IRQ routine)
		ldi TMP2,1<<PSR10					
		out SFIOR,TMP2	 	; reset prescaller for timers 0 and 1
		ldi TMP1,0xFF
		out TCNT1H,TMP1		; set HIGH byte always first!
		out TCNT1L,TMP1		; initialize TIMER1 - overflow IRQ ASAP (TNC1 = 0xFFFF)
		sei					; enable global IRQ
		sbi	PORTC,SYNLED	; SYNC LED light ON
GPS5:	clr	TMP1			; processing done or the tlg format doesn't match -> reset counter and start from scrap
		sts GCTR,TMP1

SYN:	sbic	PINC,SYNCB	; SYNC button not pressed (1) - nothing to do
		rjmp	SYN1
		sbrc	WSREG,SYNCBM	; button pressed (0) and memory bit not set, skip
		rjmp	SYN2			; memory bit set already, nothing to do
		sbr		WSREG, 1<<SYNCBM	; set the memory bit (to prevent repeated sync)
		rjmp SYNC			; do synchronisation
SYN1:	cbr		WSREG,1<<SYNCBM	; clear  bit memory
SYN2:	lds TMP1,WSCTR
		cpi	TMP1,1
		brne	END
		cbi	PORTC,SYNLED	; SYNC LED light OFF after 1st symbol

END:
		rjmp	LOOP	;back in main loop

;******* SEND DATA to AD985X **********
; data (4 bytes) taken from position in SRAM specified by X register
; LSB sent first, data followed by control byte
; takes 160 microsec

AD985X:
	cbi	PORTD,DDS_DATA
	cbi	PORTD,DDS_CLK
	cbi	PORTD,DDS_FR_UP	; all signals to low
	ldi	TMP3,4		; 4 bytes to send
AD3:	ld	TMP1,X+
	rcall	AD_BYTE
	dec	TMP3
	brne	AD3
.IFDEF AD9850
	clr	TMP1
.ELSE
	ldi TMP1,1	; clock multiplier active (6x)
.ENDIF
;	sbrc	WSREG,PWRDWN	; skip if power down not active
;	sbr	TMP1,0x04			; set bit 2 = POWER DOWN
	rcall	AD_BYTE		; send control byte (phase=0, power on/off, control = 0)
	sbi	PORTD,DDS_FR_UP	; UPDATE FREQ pulse
	ldi	TMP2,6
AD1:	dec	TMP2	; delay 4 microsec
	brne	AD1
	cbi	PORTD,DDS_FR_UP
	ldi	TMP2,6
AD2:	dec	TMP2	; delay 4 microsec
	brne	AD2
	cbi	PORTD,DDS_DATA	; SERLOAD low
	ret
	
;***** AD_BYTE ******		; sends TMP1 (one byte) to AD985X, LSB first (used by AD985X procedure)
AD_BYTE:	ldi	TMP2,8
AD_B1:	lsr	TMP1
	brcc	AD_B2
	sbi	PORTD,DDS_DATA	; bit=1, SERLOAD=1
	rjmp	AD_B3
AD_B2:	cbi	PORTD,DDS_DATA	; bit=0, SERLOAD=0
AD_B3:	nop
	nop
	nop
	nop
	sbi	PORTD,DDS_CLK	; W_CLK pulse 500 ns
	nop
	nop
	nop
	nop
	cbi	PORTD,DDS_CLK
	nop
	nop
	nop
	nop
	dec	TMP2
	brne	AD_B1
	ret


;******** PROCEDURE sets CW text pointer to begin of selected text ******
STXT:
	ldi	YH, HIGH(CWTXT)
	ldi YL, LOW(CWTXT)
	in	TMP1, PINC
	andi	TMP1, 0x18	; 4 texts specified by 2 input jumpers PC3, PC4 (text 0-3)
	lsr	TMP1			; divide by 4 = shift to position b1 and b2 (twice text ID)
	lsr TMP1
	add YL,TMP1			; address in EEPROM containing pointers to text
	clr TMP1			; carry flag untouched
	adc YH, TMP1
	rcall	RD_EE		; read low address of selected text from EEPROM
	sts	CWPTRL,TMP1
	adiw Y,1
	rcall	RD_EE		; read high address of selected text from EEPROM
	sts	CWPTRH,TMP1
	ret


;******** PROCEDURE reads one byte from EEPROM
RD_EE:	; input: address in Y register (R28,R29); output: byte in TMP1
	out	EEARL, YL
	out EEARH, YH
	sbi	EECR,EERE
RDEE1:	sbic	EECR,EERE
	rjmp	RDEE1
	in	TMP1,EEDR	; text position byte
	ret

;******** PROCEDURE reads block of data from EEPROM into SRAM ********
; input: X - pointer to SRAM, Y - pointer to EEPROM, TMP3 - number of bytes
EE_BLK:
	rcall	RD_EE
	st	X+,TMP1		; store to SRAM, increment pointers
	adiw Y,1
	dec	TMP3
	brne	EE_BLK
	ret


// TCNT is incremented, overflow when reaches 0x0000 (0xFFFF + 1)
// it means number of counted cycles = 65536 - preset value
// by 10 MHz clock one count every 25.6 usec (prescaler 1:256)
// CYCLE1 = symbol cycle, performed 175 times every doubleminute
// CYCLE2 = tail cycle, performed once at doubleminute end
// when CYCLE value incremented by 1, the cycle is shoter by 25.6 usec (by 10 MHz clock)

; ********* TIMER 1 overflow HANDLER ***********
; sets a flag indicating it's time to process next WPRS element; performed every 0,6826667 sec
; when the doubleminute flag set, initiate cycle counter
; sets a flag indicating end of doubleminute
TIM1_OVF:
		in	ISREG,SREG

		sbrs WSREG, DMIN
		rjmp TIM1_3
		cbr WSREG,1<<DMIN	; when DMIN flag set: clear it, set counter to zero
		clr ITMP1
		rjmp TMP1_4

TIM1_3:	lds ITMP1,WSCTR
		cpi ITMP1,174		; 175 cycles = 119.46667 sec (missing 0.53333 sec)
		breq TIM1_1			; last cycle in doubleminute - it is shorter
		inc ITMP1			; increment cycle counter
TMP1_4:	sts WSCTR,ITMP1		; save counter
		sbr WSREG,1<<STIME			; set symbol end flag
		lds ITMP2,CYCLE1H
		out TCNT1H, ITMP2	; TNCT1H must be written first
		lds ITMP2,CYCLE1L	; 
		out TCNT1L, ITMP2
		rjmp TIM1_2

TIM1_1:
		lds ITMP2,CYCLE2H
		out TCNT1H, ITMP2	; TNCT1H must be written first
		lds ITMP2,CYCLE2L	; = 0.533333 s
		out TCNT1L, ITMP2
		sbr	WSREG,1<<DMIN		; set doubleminute flag (used by next interrupt)

TIM1_2:	out	SREG,ISREG
		reti


; ********* TIMER 0 overflow HANDLER ***********
; sets a flag indicating it's time to process next morse element; performed every 2,667 msec
TIM0_OVF:
		in	ISREG,SREG
		dec	CWCLK
		brne	TIM0_1
		mov	CWCLK,CWSPD	; set new speed value
		sbr	CWREG,1<<T0EXP	; set flag - time expired
TIM0_1:	out	SREG,ISREG
		reti

; ******** INTERRUPT HANDLER FOR IRQ0 **************
; 1 Hz GPS pulses 20 usec length, rising edge valid, for clock synchronization
IRQ0X:
	in	ISREG,SREG
		// ToDo use the pulses in some way....

	out	SREG,ISREG
	reti

.NOLIST
;WSPR Band Dial freq (MHz) Tx freq (MHz)
;    160m 1.836600 	1.838000 - 1.838200
;    80m 3.592600 	3.594000 - 3.594200
;    60m 5.287200 	5.288600 - 5.288800
;    40m 7.038600 	7.040000 - 7.040200
;    30m 10.138700 	10.140100 - 10.140300
;    20m 14.095600 	14.097000 - 14.097200
;    17m 18.104600 	18.106000 - 18.106200
;    15m 21.094600 	21.096000 - 21.096200
;    12m 24.924600 	24.926000 - 24.926200
;    10m 28.124600 	28.126000 - 28.126200
;    6m 50.293000 	50.294400 - 50.294600
;    2m 144.488500 	144.489900 - 144.490100
.LIST

;******** EEPROM CONTENTS DEFINITION
.ESEG
.ORG	0x0

; frequency calculation - 32bit value = freq * 2^32 / clock freq
; example 50.050 MHz, clock 125 MHz AD9850 -> 0x66809d49, 50.0505 MHz -> 0x6680E065
; example 50.050 MHz, clock 180 MHz AD9851 -> 0x472EA61D, 50.0505 MHz -> 0x472ED4B8

ECWSPD:	.DB		50	; CW keying speed
.IFDEF AD9850
ECWSFT:	.DB		0x1B, 0x43	; frequency shift used for CW FSK keying (500 Hz by 125 MHz)
EWSHFT:	.DB		50			; freq shift WSPR
.ELSE
ECWSFT:	.DB		0x9A, 0x2E	; frequency shift used for CW FSK keying (500 Hz by 180 MHz)
EWSHFT:	.DB		35			; freq shift WSPR
.ENDIF

// WSPR message - 4 symbols saved as 1 byte
// OK1DX JN69 37
// 3 3 0 2 0 2 2 2 3 2 2 0 3 1 1 2 0 0 3 2 0 3 2 1 3 1 3 2 2 0
// 0 0 2 0 1 0 0 1 2 3 2 2 2 2 0 2 1 2 3 3 0 0 3 1 0 1 2 0 0 1
// 1 0 1 2 0 2 0 3 3 2 1 0 1 0 1 2 1 0 0 1 2 0 1 0 3 3 0 0 2 3
// 1 2 1 2 3 2 0 2 3 0 0 2 0 2 1 0 0 3 2 0 3 1 3 2 3 3 0 0 3 1
// 0 3 2 0 0 1 1 3 2 0 2 2 0 1 2 1 0 2 3 3 0 2 2 2 2 0 0 1 3 0
// 1 2 3 3 0 2 0 1 1 2 2 0

WSPRM:	.DB	0xf2, 0x2a, 0xe8, 0xd6, 0x0e, 0x39, 0xde, 0x80, 0x84, 0x1b, 0xaa, 0x26, 0xf00, 0xd1, 0x81, 0x46, 0x23, 0xe4, 0x46, 0x41, 0x84, 0xf0, 0xb6, 0x6e, 0x2c, 0x22, 0x43, 0x8d, 0xef, 0x0d, 0x38, 0x17, 0x8a, 0x19, 0x2f, 0x2a, 0x81, 0xc6, 0xf2, 0x16, 0x80	
CWTXT:	.DB		LOW(CWTXT3),HIGH(CWTXT3),LOW(CWTXT2),HIGH(CWTXT2),LOW(CWTXT1),HIGH(CWTXT1),LOW(CWTXT0),HIGH(CWTXT0)	; addresses of fixed texts

; clock timing - step 25.6 usec (10 MHz clock). CYCLE1 impact is 175x higher

;ECYCLE1: .DB 0xD5, 0x97 ; = 38869	- timing for symbol time, 10.0000 MHz clock
;ECYCLE2: .DB 0xD9, 0xAE	; = 44761	- timing for tail time, 10.0000 MHz

ECYCLE1: .DB 0xD1, 0x97 ; = 38865	- timing for symbol time, 10.0017 MHz clock
ECYCLE2: .DB 0x78, 0xAE	; = 44664	- timing for tail time, 10.0017 MHz


EFTSIZE:	.DB 10	; total 10 entries, each used for 2 minutes (total 20 minutes)
FTABLE:		; frequency table. Each entry is 5 bytes long - 1st byte is mode/band selector, 2nd-5th frequency
			; mode/band: bit 7 set = WSPR, cleared = CW, bits 0-2 band data, sent to outputs PD5-7
;	.DB 0, 		0x1D, 0xA6, 0x2E, 0x47	; CW 50.050 MHz
;	.DB 0x80, 	0x66, 0x92, 0x1C, 0x05	; WSPR 3.5941 MHz
;	.DB 0, 		0x1D, 0xA6, 0x2E, 0x47	; CW 50.050 MHz
;	.DB 0x80,	0x2A, 0xAB, 0x87, 0x47	; WSPR 50.2945 MHz
;	.DB 0, 		0x1D, 0xA6, 0x2E, 0x47	; CW 50.050 MHz
;	.DB 0x80, 	0x24, 0xEF, 0x6B, 0x0E	; WSPR 10.1402 MHz
;	.DB 0, 		0x1D, 0xA6, 0x2E, 0x47	; CW 50.050 MHz
;	.DB 0x80,	0xD6, 0x3A, 0xC0, 0x19	; WSPR 18.1061 MHz
;	.DB 0, 		0x1D, 0xA6, 0x2E, 0x47	; CW 50.050 MHz
;	.DB 0x80,	0x2A, 0xAB, 0x87, 0x47	; WSPR 50.2945 MHz

// CLOCK 180.00 mhZ
;	.DB 0, 		0xD8, 0x15, 0x20, 0x47	; CW 50.010 MHz
;	.DB 0x80,	0x2A, 0xAB, 0x87, 0x47	; WSPR 50.2945 MHz
;	.DB 0, 		0xE9, 0xB9, 0x23, 0x47	; CW 50.020 MHz
;	.DB 0x80,	0x2A, 0xAB, 0x87, 0x47	; WSPR 50.2945 MHz
;	.DB 0, 		0xFA, 0x5D, 0x27, 0x47	; CW 50.030 MHz
;	.DB 0x80,	0x2A, 0xAB, 0x87, 0x47	; WSPR 50.2945 MHz
;	.DB 0, 		0x0C, 0x02, 0x2B, 0x47	; CW 50.040 MHz
;	.DB 0x80,	0x2A, 0xAB, 0x87, 0x47	; WSPR 50.2945 MHz
;	.DB 0, 		0x1D, 0xA6, 0x2E, 0x47	; CW 50.050 MHz
;	.DB 0x80,	0x2A, 0xAB, 0x87, 0x47	; WSPR 50.2945 MHz

// clock 29.9995 MHz (179.997 MHz)
	.DB 0x80, 	0xFB, 0x97, 0x1C, 0x05	; WSPR 3.5941 MHz
	.DB 0x80, 	0x9F, 0x5B, 0xC0, 0x19	; WSPR 18.10615 MHz
	.DB 0x80, 	0xA4, 0x9C, 0x1C, 0x05	; WSPR 3.59415 MHz
	.DB 0x80, 	0x2F, 0x95, 0x1C, 0x05	; WSPR 3.59407 MHz
	.DB 0x80, 	0xD8, 0x99, 0x1C, 0x05	; WSPR 3.59412 MHz
	.DB 0x80, 	0xFB, 0x97, 0x1C, 0x05	; WSPR 3.5941 MHz
	.DB 0x80, 	0xB3, 0x3C, 0xC0, 0x19	; WSPR 18.10612 MHz
	.DB 0x80, 	0xA4, 0x9C, 0x1C, 0x05	; WSPR 3.59415 MHz
	.DB 0x80, 	0x2F, 0x95, 0x1C, 0x05	; WSPR 3.59407 MHz
	.DB 0x80, 	0xD8, 0x99, 0x1C, 0x05	; WSPR 3.59412 MHz



// section of flexible length
CWTXT0:	.DB		_O, _K, _1, _D, _X, _SL, _B, _SPC, _J, _N, _6, _9, _S, _D, _SPC, _SPC, _SPC, _TEND
CWTXT1:	.DB		_O, _K, _1, _D, _X, _SL, _B, _SPC, _J, _N, _6, _9, _N, _O, _SPC, _SPC, _SPC, _TEND
CWTXT2:	.DB		_O, _K, _1, _D, _X, _SPC, _B, _E, _A, _C, _O, _N, _SPC, _J, _N, _6, _9, _S, _D, _SPC, _SPC, _SPC, _TEND
CWTXT3:	.DB		_O, _K, _0, _B, _M, _SL, _B, _SPC, _J, _N, _6, _9, _S, _D, _SPC, _SPC, _SPC, _TEND
